@@ -5,6 +5,7 @@ use strict;
 use warnings;
 use Crypt::OpenSSL::X509();
 use Crypt::OpenSSL::RSA();
+use Convert::ASN1();
 use Digest();
 use MIME::Base64();
 use English qw( -no_match_vars );
@@ -20,7 +21,7 @@ our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK, );
 sub _CERTIFICATE_HEADER_SIZE { return 40; }
 sub _MAX_PUBLIC_KEY_SIZE     { return 65_536; }
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 sub pin_sha256 {
     my ($path) = @_;
@@ -33,15 +34,19 @@ sub pin_sha256 {
         /^[-]{5}BEGIN[ ](?:X[.]?509[ ]|TRUSTED[ ])?CERTIFICATE[-]{5}/smx )
     {
         $pem_encoded_public_key_string =
-          _process_certificate( $handle, $file_header, $path );
+          _process_pem_certificate( $handle, $file_header, $path );
     }
     elsif ( $file_header =~ /^[-]{5}BEGIN[ ](?:RSA[ ])?PUBLIC[ ]KEY[-]{5}/smx )
     {
         $pem_encoded_public_key_string =
-          _process_public_key( $handle, $file_header, $path );
+          _process_pem_public_key( $handle, $file_header, $path );
     }
     else {
-        Carp::croak("$path is not a PEM encoded SSL Certificate");
+        $pem_encoded_public_key_string =
+          _check_for_der_certificate( $handle, $file_header, $path );
+        if ( !defined $pem_encoded_public_key_string ) {
+            Carp::croak("$path is not an X.509 Certificate");
+        }
     }
 
     $pem_encoded_public_key_string =~
@@ -57,7 +62,7 @@ sub pin_sha256 {
     return $base64;
 }
 
-sub _process_certificate {
+sub _process_pem_certificate {
     my ( $handle, $file_header, $path ) = @_;
     my $pem_encoded_public_key_string;
     if ( $file_header =~ /^[-]{5}BEGIN[ ]CERTIFICATE[-]{5}/smx ) {
@@ -85,7 +90,21 @@ s/^([-]{5}BEGIN[ ])(?:X[.]?509|TRUSTED)[ ](CERTIFICATE[-]{5})/$1$2/smx;
     return $pem_encoded_public_key_string;
 }
 
-sub _process_public_key {
+sub _check_for_der_certificate {
+    my ( $handle, $file_header, $path ) = @_;
+    my $pem_encoded_public_key_string;
+    eval {
+        my $x509 = Crypt::OpenSSL::X509->new_from_file( $path,
+            Crypt::OpenSSL::X509::FORMAT_ASN1() );
+        $pem_encoded_public_key_string =
+          _get_pem_encoded_public_key_string($x509);
+    } or do {
+        return;
+    };
+    return $pem_encoded_public_key_string;
+}
+
+sub _process_pem_public_key {
     my ( $handle, $file_header, $path ) = @_;
     my $pem_encoded_public_key_string;
     if ( $file_header =~ /^[-]{5}BEGIN[ ]PUBLIC[ ]KEY[-]{5}/smx ) {
@@ -130,7 +149,7 @@ HTTP::PublicKeyPins - Generate RFC 7469 HTTP Public Key Pin (HPKP) header values
 
 =head1 VERSION
 
-Version 0.06
+Version 0.07
 
 =head1 SYNOPSIS
 
@@ -157,7 +176,7 @@ This module allows the calculation of RFC 7469 HTTP Public Key Pin header values
 
 =head2 pin_sha256
 
-This function accepts the path to a PEM encoded certificate.  It will load the public key from the certificate and prepare the appropriate value for the pin_sha256 parameter of the Public-Key-Pins value. This function will also make an attempt to read public keys (in PEM format).  Patches for other formats are encouraged.
+This function accepts the path to a L<X.509 Certificate|https://en.wikipedia.org/wiki/X.509>.  It will load the public key from the certificate and prepare the appropriate value for the pin_sha256 parameter of the Public-Key-Pins value. This function will also make an attempt to read public keys (in PEM format).  Patches for other formats are encouraged.
 
 =head1 SUBROUTINES/METHODS
 
@@ -169,18 +188,17 @@ None.  This module only has the one exported function.
  
 =item C<< Failed to open %s for reading >>
  
-Failed to open the supplied SSL Certificate file
+Failed to open the supplied X.509 Certificate file
  
 =item C<< Failed to read from %s >>
  
-Failed to read from the supplied SSL Certificate file
+Failed to read from the supplied X.509 Certificate file
 
-=item C<< %s is not a PEM encoded SSL Certificate >>
+=item C<< %s is not an X.509 Certificate >>
  
-The supplied input file does not look like a SSL Certificate File. An SSL Certificate file has the following header
+The supplied input file does not look like X.509 Certificate File. The X.509 Certificate file may be encoded in PEM or DER format. A PEM encoded X.509 Certificate file has the following header
 
   -----BEGIN CERTIFICATE-----
-  MII
 
 =back
  
@@ -202,7 +220,13 @@ None known.
 
 =head1 SEE ALSO
 
-L<RFC 7469 - Public Key Pinning Extension for HTTP|http://tools.ietf.org/html/rfc7469>
+=over
+
+=item L<RFC 7469 - Public Key Pinning Extension for HTTP|http://tools.ietf.org/html/rfc7469>
+
+=item L<X.509 Certificate|https://en.wikipedia.org/wiki/X.509>
+
+=back
 
 =head1 AUTHOR
 
